import process from 'node:process';

import { readKey } from '../common/fns.ts';
import { ITerminal, ITerminalSize } from '../common/types.ts';

const TsxTerminalIO: ITerminal = {
	argv: (process.argv.length > 2) ? process.argv.slice(2) : [],
	inputLoop: async function* (): AsyncGenerator<Uint8Array, null> {
		yield (await TsxTerminalIO.readStdinRaw()) ?? new Uint8Array(0);

		return null;
	},
	getTerminalSize: function getTerminalSize(): Promise<ITerminalSize> {
		const [cols, rows] = process.stdout.getWindowSize();

		return Promise.resolve({
			rows,
			cols,
		});
	},
	readStdin: async function (): Promise<string | null> {
		const raw = await TsxTerminalIO.readStdinRaw();
		return readKey(raw ?? new Uint8Array(0));
	},
	readStdinRaw: function (): Promise<Uint8Array | null> {
		return new Promise((resolve) => {
			process.stdin.resume().once(
				'data',
				(buffer: Uint8Array) => {
					resolve(buffer);
				},
			);
		});
	},
	writeStdout: function (s: string): Promise<void> {
		process.stdout.write(s);
		return Promise.resolve();
	},
};

export default TsxTerminalIO;
